unit FileObjPropSheetUnit;

interface

{$I SxInclude.inc}

uses
    Windows,
    Messages,
    SysUtils,
    Classes,
    {$IFNDEF LESS_THAN_D6}
    Variants,
    MaskUtils,
    {$ELSE}
    Mask,
    {$ENDIF}
    Forms,
    SxPropertySheetForm, Controls, StdCtrls, SxNamespacePropSheet,
    ExMegaDemoServer_TLB, ExtCtrls, SxTools, Graphics, ShellApi, AxCtrls;

type
  TGrainPropSheetForm = class(TSxShellPropSheetForm)
    edObjName: TEdit;
    Label1: TLabel;
    cbReadOnly: TCheckBox;
    imgIcon: TImage;
    Label2: TLabel;
    Bevel1: TBevel;
    Label3: TLabel;
    Bevel2: TBevel;
    Label4: TLabel;
    edtObjsCount: TEdit;
    Label5: TLabel;
    edtSize: TEdit;
    btnChangeIcon: TButton;
    btnUseSystem: TButton;
    procedure SxShellPropSheetFormCreate(Sender: TObject);
    procedure SxShellPropSheetFormShow(Sender: TObject);
    procedure btnChangeIconClick(Sender: TObject);
    procedure btnUseSystemClick(Sender: TObject);
    procedure cbReadOnlyClick(Sender: TObject);
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure LoadProperties;
    procedure SaveProperties;
  end;

implementation

{$R *.DFM}

uses
  SxProvider, PermanentModuleUnit, ComObj, ActiveX;

function FormatSizeStr(Str:String):String;
begin
  Result:=FormatMaskText('!### ### ### KB;0;*',Str);
end;

procedure TGrainPropSheetForm.LoadProperties;
var
  tempFileStg:IMDFileObj;
  tempGrain:TSxCustomDataGrain;
  tempServer:IMDServer;
begin
  tempServer:=PermanentModule.MDServer;
  tempGrain:=TSxNamespacePropertySheet(PropSheetComponent).DataGrain;
  tempServer.GetRelativeFileStorage(tempGrain.Strings.Values[vGrainPath],tempFileStg);
  if tempFileStg=nil then
     Exit;
  edObjName.Text:=tempFileStg.Name;
  edtObjsCount.Text:=IntToStr(tempFileStg.ChildCount);
  edtSize.Text:=SxFormatSizeStr(tempFileStg.Size);
  cbReadOnly.Checked:=tempFileStg.ReadOnly;
  tempGrain.GetCurrentIcon(imgIcon.Picture.Icon,is32x32,False);
  if tempFileStg.IsSystemIcon then
  begin
    btnUseSystem.Enabled:=False;
    btnChangeIcon.Enabled:=True;
  end
  else
  begin
    btnUseSystem.Enabled:=True;
    btnChangeIcon.Enabled:=False;
  end;
end;

procedure TGrainPropSheetForm.SaveProperties;
var
  tempFileStg:IMDFileObj;
  tempGrain:TSxCustomDataGrain;
  tempServer:IMDServer;
  tempPicStream:TMemoryStream;
  tempVar:OleVariant;
  tempGrainParent: TSxCustomDataGrain;
  updatedGrain: TSxCustomDataGrain;
begin
  tempServer:=CoMDServer.Create;
  tempGrain:=TSxNamespacePropertySheet(PropSheetComponent).DataGrain;
  tempServer.GetRelativeFileStorage(tempGrain.Strings.Values[vGrainPath],tempFileStg);
  if tempFileStg=nil then
     Exit;
  tempFileStg.ReadOnly:=cbReadOnly.Checked;
  if not btnUseSystem.Enabled then
     tempFileStg.IsSystemIcon:=True // currently specified system icon
  else
  begin // uploading new icon on the server
    tempPicStream:=TMemoryStream.Create;
    try
      imgIcon.Picture.Icon.SaveToStream(tempPicStream);
      tempVar:=SxStreamToVarArray(tempPicStream);
      tempFileStg.SetIcon(tempVar);
      tempFileStg.SetOpenIcon(tempVar);
    finally
      FreeAndNil(tempPicStream);
    end;
    tempFileStg.IsSystemIcon:=False; // currently specified custom icon
  end;
  tempFileStg.UpdateChanges;

  // Updating properties changes in DataGrain object
  tempGrainParent:=tempGrain.CreateParent;
  try
    updatedGrain:=PermanentModule.CreateGrainFromStg(tempFileStg, tempGrainParent);
    tempGrain.Assign(updatedGrain);
  finally
    FreeAndNil(tempGrainParent);
  end;
end;

procedure TGrainPropSheetForm.SxShellPropSheetFormCreate(Sender: TObject);
begin
  LoadProperties;
end;

procedure TGrainPropSheetForm.SxShellPropSheetFormShow(Sender: TObject);
begin
  cbReadOnly.SetFocus;
end;

procedure TGrainPropSheetForm.btnChangeIconClick(Sender: TObject);
var
  wcStr:WideString;
  tempIcon:TIcon;
  tempIconIndex:Integer;
begin
  tempIconIndex:=SxPickIconDlg(wcStr,Handle);
  if tempIconIndex<>-1 then
  begin
    tempIcon:=TIcon.Create;
    try
      tempIcon.Handle:=SxExtractIcon(wcStr,tempIconIndex);
      imgIcon.Picture.Icon.Assign(tempIcon);
      btnUseSystem.Enabled:=True;
    finally
      FreeAndNil(tempIcon);
    end;
    PageModified;
  end;
end;

procedure TGrainPropSheetForm.btnUseSystemClick(Sender: TObject);
var
  tempGrain:TSxCustomDataGrain;
begin
  btnUseSystem.Enabled:=False;
  tempGrain:=TSxNamespacePropertySheet(PropSheetComponent).DataGrain;
  tempGrain.GetCurrentIcon(imgIcon.Picture.Icon,is32x32,False);
  PageModified;
end;

procedure TGrainPropSheetForm.cbReadOnlyClick(Sender: TObject);
begin
  PageModified;
end;

initialization
    RegisterPropSheetForm(TGrainPropSheetForm);
end.












